==> eyflutter_uikit-0.0.8%2B8.tar.gz: lib/uikit/flow/flow_widget.dart <==
import 'package:eyflutter_core/kit/utils/set/list_extention.dart';
import 'package:eyflutter_uikit/uikit/flow/flow_item_widget.dart';
import 'package:eyflutter_uikit/uikit/flow/flow_provider.dart';
import 'package:flutter/material.dart';

mixin OnFlowController {
  /// 子组件单击触发
  void onFlowItemClick(FlowItem flowItem);

  /// 子组件绑定触发
  void onFlowItemBind(FlowItem flowItem, Map<String, int> idsMap);

  void onFlowBuild(OnFlowWidgetController controller);
}

mixin OnFlowWidgetController {
  /// 通知启用状态
  /// [skuIds] 启用sku id
  /// [currSkuValue] 当前选择sku
  void notifyEnable(List<Set<String>> skuIds, String currSkuValue);

  /// 获取已选中项
  List<FlowItem> getSelectedItems();

  /// 通知选项
  void notifyItems(List<FlowItem> items);
}

class FlowWidget extends StatefulWidget {
  final List<FlowItem> items;

  /// 组件内边距
  final EdgeInsetsGeometry padding;

  /// 组件背景颜色
  final Color color;

  /// 组件监听与控制器
  final OnFlowController? controller;

  /// 元素左右间距
  final double spacing;

  /// 元素上下间距
  final double runSpacing;

  /// 元素高度
  final double itemHeight;

  /// 是否单选
  final bool isSingleCheck;

  /// 元素对齐方式
  final AlignmentGeometry alignment;

  /// 超出容器启用滚动
  final scrollEnable;

  /// 边框圆角
  final BorderRadiusGeometry borderRadius;

  /// 是否允许反向选择
  final bool allowReverseChoice;

  /// 是否禁用选中样式(默认false)
  final bool disableCheckStyle;

  /// 元素背景颜色
  final Color? itemBackgroundColor;

  /// 元素选中背景颜色
  final Color? itemSelectBackgroundColor;

  /// 元素边框颜色
  final Color? itemBorderColor;

  /// 元素选中边框颜色
  final Color? itemSelectBorderColor;

  /// 元素禁用背景颜色
  final Color? itemDisableBackgroundColor;

  /// 文本颜色
  final Color? textColor;

  /// 文本大小
  final double? textSize;

  /// 选中文本颜色
  final Color? selectTextColor;

  const FlowWidget({
    Key? key,
    required this.items,
    this.padding = EdgeInsets.zero,
    this.color = Colors.white,
    this.controller,
    this.spacing = 6,
    this.runSpacing = 6,
    this.itemHeight = 32,
    this.isSingleCheck = true,
    this.scrollEnable = false,
    this.alignment = Alignment.topLeft,
    this.borderRadius = const BorderRadius.all(Radius.circular(3)),
    this.allowReverseChoice = true,
    this.disableCheckStyle = false,
    this.itemBackgroundColor,
    this.itemSelectBackgroundColor,
    this.itemBorderColor,
    this.itemSelectBorderColor,
    this.itemDisableBackgroundColor,
    this.textColor,
    this.textSize,
    this.selectTextColor,
  }) : super(key: key);

  @override
  _FlowWidgetState createState() => _FlowWidgetState();
}

class _FlowWidgetState extends State<FlowWidget> with OnFlowWidgetController {
  Map<int, OnFlowItemController> _itemControllers = {};

  //skuId-index
  Map<String, int> idsMap = {};
  List<FlowItem> flowItems = [];
  bool checkFlag = false;

  @override
  void initState() {
    flowItems = widget.items;
    widget.controller?.onFlowBuild(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: widget.color,
      width: double.infinity,
      padding: widget.padding,
      child: ListView(
        physics: widget.scrollEnable ? const AlwaysScrollableScrollPhysics() : const NeverScrollableScrollPhysics(),
        shrinkWrap: true,
        children: [
          Align(
            alignment: widget.alignment,
            child: Wrap(
              spacing: widget.spacing,
              runSpacing: widget.runSpacing,
              alignment: WrapAlignment.start,
              runAlignment: WrapAlignment.end,
              children: _buildItems(),
            ),
          )
        ],
      ),
    );
  }

  List<Widget> _buildItems() {
    List<Widget> lst = [];
    //单选情况下，若数据源有多项选择则以第一项选中为准
    int position = 0;
    flowItems.forEach((element) {
      lst.add(_buildItem(element, position));
      widget.controller?.onFlowItemBind(element, idsMap);
      position++;
    });
    return lst;
  }

  Widget _buildItem(FlowItem flowItem, int position) {
    return FlowItemWidget(
      height: widget.itemHeight,
      text: flowItem.text,
      textColor: widget.textColor,
      textSize: widget.textSize,
      selectTextColor: widget.selectTextColor,
      itemClick: _itemClick,
      isCheck: (widget.isSingleCheck) ? (checkFlag ? false : checkFlag = (flowItem.isCheck)) : flowItem.isCheck,
      data: flowItem.data,
      changeCall: _itemChange,
      position: position,
      buildCall: (index, controller) {
        _itemControllers[index] = controller;
      },
      tagWidget: flowItem.tagWidget,
      enable: flowItem.enable,
      borderRadius: widget.borderRadius,
      backgroundColor: widget.itemBackgroundColor ?? Colors.white,
      selectedBackgroundColor: widget.itemSelectBackgroundColor ?? Colors.white,
      borderColor: widget.itemBorderColor ?? Color(0xffefefef),
      selectedBorderColor: widget.itemSelectBorderColor ?? Color(0xffFF2E18),
      disableBackgroundColor: widget.itemDisableBackgroundColor ?? Color(0xfff8f8f8),
    );
  }

  void _itemChange(int position, bool isCheck, dynamic data) {
    int pos = 0;
    flowItems.forEach((element) {
      if (pos == position) {
        if (widget.isSingleCheck) {
          element.isCheck = !isCheck;
          _notifyItemState(pos, !isCheck);
        } else {
          element.isCheck = isCheck;
          _notifyItemState(pos, isCheck);
        }
      } else {
        if (widget.isSingleCheck) {
          if (element.isCheck) {
            _notifyItemState(pos, false);
          }
          element.isCheck = false;
        }
      }
      pos++;
    });
  }

  void _notifyItemState(int index, bool check) {
    var controller = _itemControllers[index];
    if (controller == null) {
      return;
    }
    controller.notifyCheck(check);
  }

  void _itemClick(int position) {
    if (!widget.disableCheckStyle) {
      flowItems.forEachEle((index, element) {
        if (index == position) {
          if (widget.allowReverseChoice) {
            element.isCheck = !element.isCheck;
          } else {
            element.isCheck = true;
          }
        } else {
          element.isCheck = false;
        }
        _notifyItemState(index, element.isCheck);
      });
    }
    var skuItem = flowItems[position];
    widget.controller?.onFlowItemClick(skuItem);
  }

  @override
  void notifyEnable(List<Set<String>> skuIds, String currSkuValue) {
    int pos = 0;
    flowItems.forEach((element) {
      var controller = _itemControllers[pos];
      if (controller != null) {
        var msValue = currSkuValue.split("_");
        var refsValue = element.specSkuMapping?.split("_") ?? [];
        //同一规格下sku不作处理
        if (msValue.isNotEmpty && refsValue.isNotEmpty) {
          if (msValue[0] != refsValue[0]) {
            if (hasEffectiveValue(skuIds, element.specSkuMapping ?? "")) {
              element.enable = true;
            } else {
              element.enable = false;
            }
            controller.notifyEnable(element.enable);
          } else {
            element.enable != element.enable;
            controller.notifyEnable(element.enable);
          }
        }
      }
      pos++;
    });
  }

  bool hasEffectiveValue(List<Set<String>> skuIds, String refValue) {
    bool flag = false;
    skuIds.forEach((sets) {
      if (sets.contains(refValue)) {
        flag = true;
        return;
      }
    });
    return flag;
  }

  @override
  List<FlowItem> getSelectedItems() {
    List<FlowItem> lst = [];
    flowItems.forEach((element) {
      if (element.isCheck) {
        lst.add(element);
      }
    });
    return lst;
  }

  @override
  void notifyItems(List<FlowItem> items) {
    setState(() {
      flowItems = items;
    });
  }
}

==> eyflutter_uikit-0.1.4%2B8.tar.gz: lib/uikit/flow/flow_widget.dart <==
import 'package:eyflutter_core/kit/utils/set/list_extention.dart';
import 'package:eyflutter_core/provider/cus_change_notifier.dart';
import 'package:eyflutter_uikit/uikit/flow/flow_item_widget.dart';
import 'package:eyflutter_uikit/uikit/flow/flow_provider.dart';
import 'package:flutter/material.dart';

mixin OnFlowController {
  /// 子组件单击触发
  void onFlowItemClick(FlowItem flowItem);

  /// 子组件绑定触发
  void onFlowItemBind(FlowItem flowItem);

  void onFlowBuild(OnFlowWidgetController controller);
}

mixin OnFlowWidgetController {
  /// 通知启用状态
  /// [skuIds] 启用sku id
  /// [currSkuValue] 当前选择sku
  void notifyEnable(List<Set<String>> skuIds, String currSkuValue);

  /// 获取已选中项
  List<FlowItem> getSelectedItems();

  /// 通知选项
  void notifyItems(List<FlowItem> items);

  /// 列表滚动控制器
  ScrollController scrollController = ScrollController();
}

class FlowWidget extends StatefulWidget {
  final List<FlowItem> items;

  /// 组件内边距
  final EdgeInsetsGeometry padding;

  /// 组件背景颜色
  final Color color;

  /// 组件监听与控制器
  final OnFlowController? controller;

  /// 元素左右间距
  final double spacing;

  /// 元素上下间距
  final double runSpacing;

  /// 元素宽度
  final double? itemWidth;

  /// 元素高度
  final double itemHeight;

  /// 是否单选
  final bool isSingleCheck;

  /// 元素对齐方式
  final AlignmentGeometry alignment;

  /// 超出容器启用滚动
  final scrollEnable;

  /// 边框圆角
  final BorderRadiusGeometry borderRadius;

  /// 是否允许反向选择
  final bool allowReverseChoice;

  /// 是否禁用选中样式(默认false)
  final bool disableCheckStyle;

  /// 元素背景颜色
  final Color? itemBackgroundColor;

  /// 元素选中背景颜色
  final Color? itemSelectBackgroundColor;

  /// 元素边框颜色
  final Color? itemBorderColor;

  /// 元素选中边框颜色
  final Color? itemSelectBorderColor;

  /// 元素禁用背景颜色
  final Color? itemDisableBackgroundColor;

  /// 文本颜色
  final Color? textColor;

  /// 文本大小
  final double? textSize;

  /// 选中文本颜色
  final Color? selectTextColor;

  /// 选项内边距
  final EdgeInsetsGeometry? itemPadding;

  /// 是否反选
  final bool reverse;

  const FlowWidget({
    Key? key,
    required this.items,
    this.padding = EdgeInsets.zero,
    this.color = Colors.white,
    this.controller,
    this.spacing = 6,
    this.runSpacing = 6,
    this.itemWidth,
    this.itemHeight = 32,
    this.isSingleCheck = true,
    this.scrollEnable = false,
    this.alignment = Alignment.topLeft,
    this.borderRadius = const BorderRadius.all(Radius.circular(3)),
    this.allowReverseChoice = true,
    this.disableCheckStyle = false,
    this.itemBackgroundColor,
    this.itemSelectBackgroundColor,
    this.itemBorderColor,
    this.itemSelectBorderColor,
    this.itemDisableBackgroundColor,
    this.textColor,
    this.textSize,
    this.selectTextColor,
    this.itemPadding,
    this.reverse = false,
  }) : super(key: key);

  @override
  FlowWidgetState createState() => FlowWidgetState();
}

class FlowModel extends CusChangeNotifier {}

class FlowItemEntry {
  OnFlowItemController controller;
  FlowItemModel model;

  FlowItemEntry({required this.controller, required this.model});
}

class FlowWidgetState extends State<FlowWidget> with OnFlowWidgetController {
  Map<int, FlowItemEntry> _itemElements = {};

  List<FlowItem> flowItems = [];
  bool checkFlag = false;

  @override
  void initState() {
    flowItems = widget.items;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    widget.controller?.onFlowBuild(this);
    return Container(
      color: widget.color,
      width: double.infinity,
      padding: widget.padding,
      child: ListView(
        padding: EdgeInsets.zero,
        physics: widget.scrollEnable ? const AlwaysScrollableScrollPhysics() : const NeverScrollableScrollPhysics(),
        shrinkWrap: true,
        controller: this.scrollController,
        reverse: widget.reverse,
        children: [
          Align(
            alignment: widget.alignment,
            child: Wrap(
              spacing: widget.spacing,
              runSpacing: widget.runSpacing,
              alignment: WrapAlignment.start,
              runAlignment: WrapAlignment.end,
              children: _buildItems(),
            ),
          )
        ],
      ),
    );
  }

  List<Widget> _buildItems() {
    List<Widget> lst = [];
    //单选情况下，若数据源有多项选择则以第一项选中为准
    int position = 0;
    flowItems.forEach((element) {
      lst.add(_buildItem(element, position));
      widget.controller?.onFlowItemBind(element);
      position++;
    });
    return lst;
  }

  Widget _buildItem(FlowItem flowItem, int position) {
    return FlowItemWidget(
      key: ValueKey(flowItem.hashCode),
      width: widget.itemWidth,
      height: widget.itemHeight,
      text: flowItem.text,
      textColor: widget.textColor,
      textSize: widget.textSize,
      selectTextColor: widget.selectTextColor,
      itemClick: _itemClick,
      isCheck: (widget.isSingleCheck) ? (checkFlag ? false : checkFlag = (flowItem.isCheck)) : flowItem.isCheck,
      data: flowItem.data,
      changeCall: _itemChange,
      position: position,
      buildCall: (index, controller, FlowItemModel itemModel) {
        _itemElements[index] = FlowItemEntry(controller: controller, model: itemModel);
      },
      tagWidget: flowItem.tagWidget,
      enable: flowItem.enable,
      borderRadius: widget.borderRadius,
      backgroundColor: widget.itemBackgroundColor ?? Colors.white,
      selectedBackgroundColor: widget.itemSelectBackgroundColor ?? Colors.white,
      borderColor: widget.itemBorderColor ?? Color(0xffefefef),
      selectedBorderColor: widget.itemSelectBorderColor ?? Color(0xffFF2E18),
      disableBackgroundColor: widget.itemDisableBackgroundColor ?? Color(0xfff8f8f8),
      padding: widget.itemPadding ?? EdgeInsets.zero,
    );
  }

  void _itemChange(int position, bool isCheck, dynamic data) {
    int pos = 0;
    flowItems.forEach((element) {
      if (pos == position) {
        if (widget.isSingleCheck) {
          element.isCheck = !isCheck;
          _notifyItemState(pos, !isCheck);
        } else {
          element.isCheck = isCheck;
          _notifyItemState(pos, isCheck);
        }
      } else {
        if (widget.isSingleCheck) {
          if (element.isCheck) {
            _notifyItemState(pos, false);
          }
          element.isCheck = false;
        }
      }
      pos++;
    });
  }

  void _notifyItemState(int index, bool check) {
    var entry = _itemElements[index];
    if (entry == null) {
      return;
    }
    entry.controller.notifyCheck(check);
  }

  void _itemClick(int position) {
    if (!widget.disableCheckStyle) {
      flowItems.forEachEle((index, element) {
        if (index == position) {
          if (widget.allowReverseChoice) {
            element.isCheck = element.isCheck;
          } else {
            element.isCheck = true;
          }
        } else {
          if (widget.isSingleCheck) {
            element.isCheck = false;
          }
        }
        _notifyItemState(index, element.isCheck);
      });
    }
    var skuItem = flowItems[position];
    widget.controller?.onFlowItemClick(skuItem);
  }

  @override
  void notifyEnable(List<Set<String>> skuIds, String currSkuValue) {
    int pos = 0;
    flowItems.forEach((element) {
      var entry = _itemElements[pos];
      if (entry != null) {
        var msValue = currSkuValue.split("_");
        var refsValue = element.specSkuMapping?.split("_") ?? [];
        //同一规格下sku不作处理
        if (msValue.isNotEmpty && refsValue.isNotEmpty) {
          if (msValue[0] != refsValue[0]) {
            if (hasEffectiveValue(skuIds, element.specSkuMapping ?? "")) {
              element.enable = true;
            } else {
              element.enable = false;
            }
            entry.controller.notifyEnable(element.enable);
          } else {
            element.enable != element.enable;
            entry.controller.notifyEnable(element.enable);
          }
        }
      }
      pos++;
    });
  }

  bool hasEffectiveValue(List<Set<String>> skuIds, String refValue) {
    bool flag = false;
    skuIds.forEach((sets) {
      if (sets.contains(refValue)) {
        flag = true;
        return;
      }
    });
    return flag;
  }

  @override
  List<FlowItem> getSelectedItems() {
    List<FlowItem> lst = [];
    flowItems.forEach((element) {
      if (element.isCheck) {
        lst.add(element);
      }
    });
    return lst;
  }

  @override
  void notifyItems(List<FlowItem> items) {
    if (mounted) {
      setState(() {
        flowItems = items;
      });
    }
  }
}
